/* avgw.c (weighted average) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file avgw.c
 * @brief Weighted average.
 *
 * CREATE AGGREGATE FUNCTION avgw RETURNS REAL SONAME 'libsqlstat.so';
 * SELECT avgw(x, w) FROM mytab;
 * DROP FUNCTION avgw;
 * */

#include "sqlstat.h"

/**
 * @brief Storage for weighted average.
 */
struct avgw_storage {
  int     argc;  /**< @brief number of arguments */
  double  weight;  /**< @brief Sum of weight. */
  double  sumX;   /**< @brief Sum of x */
};

/**
 * \fn my_bool avgw_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 * @brief Called before first usage of function.
 *
 * Check parameters.<br />
 * Indicate type of return value.<br />
 * Allocate memory for sums.
 * @param initid function description
 * @param args parameters
 * @param message error message
 * @return success = 0
 */
my_bool avgw_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
  struct avgw_storage * data;
  
  if (args->arg_count < 1 || args->arg_count > 2) {
    strcpy(message,"avgw() requires one or two arguments");
    return 1;
  }
  args->arg_type[0] = REAL_RESULT;
  if (args->arg_count > 1) {
    args->arg_type[1] = REAL_RESULT;
  }

  data = (struct avgw_storage *) malloc( sizeof(struct avgw_storage));
  if (data == NULL) {
    strcpy(message,"Couldn't allocate memory");
    return 1;
  }  
  data->argc = args->arg_count;
  
  initid->maybe_null = 1;
  initid->decimals   = NOT_FIXED_DEC;
  initid->max_length = 13 + initid->decimals;
  initid->ptr        = (char *) data;
  initid->const_item = 0;
  
  return 0;
}

/**
 * \fn void avgw_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Reset function and add first group member
 * Calls clear and add.
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 * @deprecated Reset is only called by MySQL 4.0 and below. It is implemented for backwards
 * compatibility.
 */
void avgw_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  avgw_clear(initid, is_null, error);
  avgw_add(initid, args, is_null, error);
}

/**
 * void avgw_clear(UDF_INIT *initid, char *is_null, char *error)
 * @brief Called at start of group.
 *
 * The sums are cleared. 
 * @param initid function description
 * @param is_null
 * @param error error message
 */
void avgw_clear(UDF_INIT *initid, char *is_null, char *error) {
  struct avgw_storage *data;
  data = (struct avgw_storage *) initid->ptr;
  data->weight = 0;
  data->sumX  = 0;
}

/**
 * \fn void avgw_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Add a member of the group.
 *
 * The sums are updated
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 */
void avgw_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  struct avgw_storage * data;
  double x;
  double w;
  
  if (!args->args[0]) {
    return;
  }
  data = (struct avgw_storage *) initid->ptr;
  if (data->argc > 1) {
    if (!args->args[1]) {
      return;
    }
    w = *((double*) args->args[1]);
  } else {
    w = 1.;
  }
  x = *((double*) args->args[0]);
  
  data->weight  += w;
  data->sumX   += w * x;
}

/**
 * \fn double avgw(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Retrieve weighted average. Called at end of group.
 * @param initid function description
 * @param args argument values
 * @param is_null result is NULL
 * @param error error message
 * @return weighted average
 */
double avgw(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  struct avgw_storage * data;
  
  data = (struct avgw_storage *) initid->ptr;

  if (data->weight <= 0) {
    *is_null = 1;
    return 0;
  }
  
  return data->sumX / data->weight;
}

/**
 * \fn void avgw_deinit(UDF_INIT *initid)
 * @brief Called after last access to function.
 *
 * Deallocate memory for counter and sums.
 * @param initid
 */
void avgw_deinit(UDF_INIT *initid) {
  if (initid->ptr) {
    free(initid->ptr);
  }
}
