/* median.cc (median) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/


/**
 * @file median.cc
 * @brief Median
 *
 * CREATE AGGREGATE FUNCTION median RETURNS REAL SONAME 'libsqlstat.so';<br />
 * SELECT median(x) FROM mytab;<br />
 * DROP FUNCTION median;<br />
 * */

#include "sqlstat.h"
#include <set>

using namespace std;

/**
 * @brief Multiset of doubles
 */
typedef multiset<double, less<double> > DoubleSet;

/**
 * \fn my_bool median_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
 * @brief Called before first usage of function.
 *
 * Check parameters.<br />
 * Indicate type of return value.<br />
 * Allocate memory for counter and sums.
 * @param initid function description
 * @param args parameters
 * @param message error message
 * @return success = 0
 */
my_bool median_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
  DoubleSet *data;
  
  if (args->arg_count != 1) {
    strcpy(message,"median() requires one argument");
    return 1;
  }
  args->arg_type[0] = REAL_RESULT;

  data = new DoubleSet();
  if (data == NULL) {
    strcpy(message,"Couldn't allocate memory");
    return 1;
  }  
  
  initid->maybe_null = 1;
  initid->decimals   = NOT_FIXED_DEC;
  initid->max_length = 13 + initid->decimals;
  initid->ptr        = (char *) data;
  initid->const_item = 0;
  
  return 0;
}

/**
 * \fn void median_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Reset function and add first group member
 * Calls clear and add.
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 * @deprecated Reset is only called by MySQL 4.0 and below. It is implemented for backwards
 * compatibility.
 */
void median_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  median_clear(initid, is_null, error);
  median_add(initid, args, is_null, error);
}

/**
 * \fn void median_clear(UDF_INIT *initid, char *is_null, char *error)
 * @brief Called at start of group.
 *
 * The counter and sums are cleared. 
 * @param initid function description
 * @param is_null
 * @param error error message
 */
void median_clear(UDF_INIT *initid, char *is_null, char *error) {
  DoubleSet *data;

  data = (DoubleSet *) initid->ptr;
  data->clear();
}

/**
 * \fn void median_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Add a member of the group.
 *
 * The counter and sums are updated
 * @param initid function description
 * @param args argument values
 * @param is_null
 * @param error error message
 */
void median_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  DoubleSet *data;
  double x;
  
  if (!args->args[0]) {
    return;
  }
  data = (DoubleSet *) initid->ptr;
  x = *((double*) args->args[0]);

  data->insert(x);
}

/**
 * \fn double median(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error)
 * @brief Retrieve median. Called at end of group.
 * @param initid function description
 * @param args argument values
 * @param is_null result is NULL
 * @param error error message
 * @return median
 */
double median(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error) {
  DoubleSet *data;
  DoubleSet::iterator pos;
  double ret;
  size_t n;
  size_t n2;

  data = (DoubleSet *) initid->ptr;

  
  n = data->size();
  if (n <= 0 ) {
    *is_null = 1;
    return 0;
  }

  n2 = ++n / 2;

  pos = data->begin();
  for (int i = 0; i < n2; i++) {
    ret = *pos++;
  }

  if (1 & n) {
    ret += *pos;
    ret /= 2;
  }
  
  return ret;
}

/**
 * \fn median_deinit(UDF_INIT *initid)
 * @brief Called after last access to function.
 *
 * Deallocate memory for counter and sums.
 * @param initid
 */
void median_deinit(UDF_INIT *initid) {
  DoubleSet *data;
  if (initid->ptr) {
    data = (DoubleSet *) initid->ptr;
    data->~multiset();
  }
}
