/* sqlstat.h (Statistics for MySQL) */

/***********************************************************************
*  This code is part of Statistics for MySQL.
*
*  Copyright (C) 2011 Heinrich Schuchardt (xypron.glpk@gmx.de)
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
***********************************************************************/

/**
 * @file sqlstat.h
 * @brief Definition of functions for UDFs and plugins.
 */

/** 
 *  @mainpage 
 *  
 *  @section sec1 Introduction 
 *  Statistics for MySQL provides additional statistical functions (like
 *  Pearson's coefficient of correlation) which are missing in MySQL.
 *
 *  The project homepage is <a href="http://sqlstat.sourceforge.net"
 *  >http://sqlstat.sourceforge.net</a>.
 *
 *  @section sec2 Download
 *  The latest release of "Statistics for MySQL" is available at
 *  <a href="http://sourceforge.net/projects/sqlstat/"
 *  >http://sourceforge.net/projects/sqlstat/</a>.
 *
 *  You can download the latest development version from the subversion
 *  repository with
 *  <pre>
 *  svn co https://sqlstat.svn.sourceforge.net/svnroot/sqlstat/libsqlstat/trunk libsqlstat</pre>
 *  or view it the repository at
 *  <a href="http://sqlstat.svn.sourceforge.net/viewvc/sqlstat/"
 *  >http://sqlstat.svn.sourceforge.net/viewvc/sqlstat/</a>.
 */


#ifndef SQLSTAT_H

#define SQLSTAT_H 1

#ifdef HAVE_CONF_H
#include "config.h"
#endif // HAVE_CONF_H

#if defined SQLWIN
#include "windows.h"
#endif //SQLWIN

#define MYSQL_DYNAMIC_PLUGIN 1

#include <mysql.h>
#include <plugin.h>
#include <malloc.h>
#include <math.h>
#include <string.h>

#ifndef NOT_FIXED_DEC
/**
 * @def NOT_FIXED_DEC
 * @brief Maximum number of digits in double
 * As defined in mysql/sql_string.h
 */
#define NOT_FIXED_DEC 31
#endif // NOT_FIXED_DEC

#ifdef __cplusplus
extern "C"
{
#endif

int sqlstat_plugin_init(void *p);
int sqlstat_plugin_deinit(void *p);

int sqlstat_plugin_isloaded();

my_bool avgw_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
void avgw_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void avgw_clear(UDF_INIT *initid, char *is_null, char *error);
void avgw_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
double avgw(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void avgw_deinit(UDF_INIT *initid);

my_bool corr_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
void corr_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void corr_clear(UDF_INIT *initid, char *is_null, char *error);
void corr_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
double corr(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void corr_deinit(UDF_INIT *initid);

my_bool median_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
void median_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void median_clear(UDF_INIT *initid, char *is_null, char *error);
void median_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
double median(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void median_deinit(UDF_INIT *initid);

my_bool rand_mt_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
double rand_mt(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void rand_mt_deinit(UDF_INIT *initid);

my_bool rand_norm_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
double rand_norm(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void rand_norm_deinit(UDF_INIT *initid);

my_bool rownumber_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
long long rownumber(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void rownumber_deinit(UDF_INIT *initid);

my_bool skewness_pop_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
void skewness_pop_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void skewness_pop_clear(UDF_INIT *initid, char *is_null, char *error);
void skewness_pop_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
double skewness_pop(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void skewness_pop_deinit(UDF_INIT *initid);

my_bool skewness_samp_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
void skewness_samp_reset(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void skewness_samp_clear(UDF_INIT *initid, char *is_null, char *error);
void skewness_samp_add(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
double skewness_samp(UDF_INIT *initid, UDF_ARGS *args, char *is_null, char *error);
void skewness_samp_deinit(UDF_INIT *initid);

#ifdef __cplusplus
}
#endif

#endif // SQLSTAT_H

